/*
Description : Universal IIC (I2C) Driver for MC9S08GB60A
              At the moment only slave capabilities are implemented.

Platform    : MC9S08GB60A (shoule run on every HS08 and maybe others)	

Author      : Erich Studer

Contact     : richu_@hotmail.com

Date        : Dez. 2010

History
	Whow    :
	What    :
	Date    :

http://www.mikrocontroller.net/topic/221676#2221070

*/


#include <hidef.h> /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */

#include "iic.h"


IIC_handlerT __IIC_handler;


unsigned char* data;
unsigned int bytesLeft;



/*********************************/
// Universal interrupt function for IIC.
// At the moment only slave mode is implemented 
//
// In:  - 
//    
// Out: -
/*********************************/
void interrupt VectorNumber_Viic1 IICinterrupt(void){
  //siehe MC9S08GB60A.pdf; Seite 221; Figure 13-11. Typical IIC Interrupt Routine
  char dummy;

  IIC1S_IICIF = 1; //interrupt flag lschen  
  if(IIC1C_MST == 1){ //if master mode
    return; //master mode wird zur Zeit nicht untersttzt  
  }else{//slave mode    
    if(IIC1S_ARBL == 1){ //IIC1S_ARBL == 1 sollte gar nie eintreten, da Master Mode nicht untersttzt wird.
      IIC1S_ARBL = 1; //Interrupt Flag lschen      
      return; //Da der Master Mode im Moment nicht untersttzt wird, macht auch Behandlung von IIC1S_ARBL==1 keinen Sinn
    }else{
      if(IIC1S_IAAS == 1){ //Addressed as a Slave
        IIC1S_IAAS = 1; //Flag lschen
        if(IIC1S_SRW == 1){//(TX) Master reading from Slave (ich bin slave)
          IIC1C_TX = 1; //set TX Mode
          /////////////////////////////////////////////
          //hier muss das erste Byte aus dem Buffer geholt werden
          //und ins IIC1D geschrieben werden
          IIC1D = *(data++);
          bytesLeft--;   
        }else{ //(RX) Master writing to Slave (ich bin slave)
          IIC1C_TX = 0; //set RX Mode
          dummy = IIC1D; //dummy read; alles Bereit fr Empfang nchstes Byte
        }
      }else{ //if not Addressed as a Slave (Datenbertragung)
        if(IIC1C_TX == 1){ //TX; from Slave to Master (ich bin slave)
          if(IIC1S_RXAK==1){//kein ACK erhalten
            IIC1C_TX = 0; //set RX Mode
            dummy = IIC1D; //dummy read; bereitmachen zum Empfang des nchsten Bytes
          }else{  //ACK erhalten
            /////////////////////////////////////////////
            //hier muss das nchste Byte aus dem Buffer geholt werden
            //und ins IIC1D geschrieben werden
            if(bytesLeft > 0){
              IIC1D = *(data++);
              bytesLeft--;
            }
          }
        }else{ //RX; from Master to Slave (ich bin slave)
           __IIC_handler(IIC1D);
        } 
      }
    }
  }   
}



/*********************************/
// Initialisation IIC
//
// In:  handler:  handler function, called for every byte received 
//    
// Out: -
/*********************************/
void IIC_init(IIC_handlerT handler){
  __IIC_handler = handler;
 
  IIC1A = 0xAA;  // IIC1A>>1 = Slave-Adresse (0xAA entspricht der Adresse 0x55)
 
  IIC1C_IICIE = 1; //Interrupt enable
  IIC1C_IICEN = 1; //enable IIC
  IIC1S_IICIF = 1; //interrupt flag lschen   
}



/*********************************/
// Sends some bytes of data 
//
// In:  dataPtr: pointer to the data
//      size   : byte size of data
//
// Out: -
/*********************************/
void IIC_send(unsigned char* dataPtr, unsigned int size){
  data = dataPtr;
  bytesLeft = size;
}

